﻿

#ifndef __ZEGOEXPRESSERRORCODE_H__
#define __ZEGOEXPRESSERRORCODE_H__

namespace ZEGO {
    namespace EXPRESS {

        enum ZegoErrorCode
        {
            /** 执行成功 */
            ZEGO_ERROR_CODE_COMMON_SUCCESS                                         = 0,

            /** 未初始化引擎，在调用非静态方法前需要先初始化引擎 */
            ZEGO_ERROR_CODE_COMMON_ENGINE_NOT_CREATE                               = 1000001,

            /** 未登陆房间, 在调用推流或拉流前，请先登陆房间 */
            ZEGO_ERROR_CODE_COMMON_NOT_LOGIN_ROOM                                  = 1000002,

            /** 引擎未启动，请先调用预览/推流/拉流接口 */
            ZEGO_ERROR_CODE_COMMON_ENGINE_NOT_STARTED                              = 1000003,

            /** 调用了当前平台不支持的接口, 例如在非Android平台调用了设置Android上下文环境的接口 */
            ZEGO_ERROR_CODE_COMMON_UNSUPPORTED_PLATFORM                            = 1000006,

            /** 无效的 Android 上下文环境 */
            ZEGO_ERROR_CODE_COMMON_INVALID_ANDROID_ENVIRONMENT                     = 1000007,

            /** 重复设置回调无效，请先将回调设置为空再尝试重新设置 */
            ZEGO_ERROR_CODE_COMMON_EVENT_HANDLER_EXISTS                            = 1000008,

            /** SDK没有包含此模块功能，请联系技术支持提供 */
            ZEGO_ERROR_CODE_COMMON_SDK_NO_MODULE                                   = 1000010,

            /** 输入流ID超长, 最大支持输入256个字符 */
            ZEGO_ERROR_CODE_COMMON_STREAM_ID_TOO_LONG                              = 1000014,

            /** 输入流ID为空 */
            ZEGO_ERROR_CODE_COMMON_STREAM_ID_NULL                                  = 1000015,

            /** 输入流ID包含非法字符 */
            ZEGO_ERROR_CODE_COMMON_STREAM_ID_INVALID_CHARACTER                     = 1000016,

            /** 该AppID已经下线 */
            ZEGO_ERROR_CODE_COMMON_APP_OFFLINE_ERROR                               = 1000037,

            /** 后台配置有错误，请检查APP配置项 */
            ZEGO_ERROR_CODE_COMMON_APP_FLEXIABLE_CONFIG_ERROR                      = 1000038,

            /** CDN地址有误，请检查支持的协议和格式 */
            ZEGO_ERROR_CODE_COMMON_CDN_URL_INVALID                                 = 1000055,

            /** DNS解析失败，请检查网络配置 */
            ZEGO_ERROR_CODE_COMMON_DNS_RESOLVE_ERROR                               = 1000060,

            /** 服务器调度异常，请联系技术支持解决 */
            ZEGO_ERROR_CODE_COMMON_DISPATCH_ERROR                                  = 1000065,

            /** 内部空指针错误，请联系技术支持解决 */
            ZEGO_ERROR_CODE_COMMON_INNER_NULLPTR                                   = 1000090,

            /** app ID不能为0，请检查app ID是否正确 */
            ZEGO_ERROR_CODE_ENGINE_APPID_ZERO                                      = 1001000,

            /** 输入AppSign长度必须为64字节 */
            ZEGO_ERROR_CODE_ENGINE_APPSIGN_INVALID_LENGTH                          = 1001001,

            /** 输入AppSign包含非法字符, 仅支持'0'-'9','a'-'f','A'-'F' */
            ZEGO_ERROR_CODE_ENGINE_APPSIGN_INVALID_CHARACTER                       = 1001002,

            /** 输入AppSign为空 */
            ZEGO_ERROR_CODE_ENGINE_APPSIGN_NULL                                    = 1001003,

            /** 认证失败，请检查app ID是否正确；或是否在没有向ZEGO申请上线的情况下选择了正式环境初始化 */
            ZEGO_ERROR_CODE_ENGINE_APPID_INCORRECT_OR_NOT_ONLINE                   = 1001004,

            /** AppSign认证失败，请检查AppSign是否正确 */
            ZEGO_ERROR_CODE_ENGINE_APPSIGN_INCORRECT                               = 1001005,

            /** 日志没有写入权限 */
            ZEGO_ERROR_CODE_ENGINE_LOG_NO_WRITE_PERMISSION                         = 1001014,

            /** 输入日志的路径超长 */
            ZEGO_ERROR_CODE_ENGINE_LOG_PATH_TOO_LONG                               = 1001015,

            /** 登陆房间数量超过上限，当前仅支持同时登陆1个房间 */
            ZEGO_ERROR_CODE_ROOM_COUNT_EXCEED                                      = 1002001,

            /** 输入用户ID为空 */
            ZEGO_ERROR_CODE_ROOM_USER_ID_NULL                                      = 1002005,

            /** 输入用户ID包含非法字符 */
            ZEGO_ERROR_CODE_ROOM_USER_ID_INVALID_CHARACTER                         = 1002006,

            /** 输入用户ID超长，最大支持输入64个字符 */
            ZEGO_ERROR_CODE_ROOM_USER_ID_TOO_LONG                                  = 1002007,

            /** 输入用户名为空 */
            ZEGO_ERROR_CODE_ROOM_USER_NAME_NULL                                    = 1002008,

            /** 输入用户名包含非法字符 */
            ZEGO_ERROR_CODE_ROOM_USER_NAME_INVALID_CHARACTER                       = 1002009,

            /** 输入用户ID超长，最大支持输入256个字符 */
            ZEGO_ERROR_CODE_ROOM_USER_NAME_TOO_LONG                                = 1002010,

            /** 输入房间ID为空 */
            ZEGO_ERROR_CODE_ROOM_ROOMID_NULL                                       = 1002011,

            /** 输入房间ID包含非法字符 */
            ZEGO_ERROR_CODE_ROOM_ROOMID_INVALID_CHARACTER                          = 1002012,

            /** 输入房间ID超长，最大支持输入128个字符 */
            ZEGO_ERROR_CODE_ROOM_ROOMID_TOO_LONG                                   = 1002013,

            /** 登陆房间失败，可能是由于网络原因 */
            ZEGO_ERROR_CODE_ROOM_ERROR_CONNECT_FAILED                              = 1002030,

            /** 登陆房间超时，可能是由于网络原因 */
            ZEGO_ERROR_CODE_ROOM_ERROR_LOGIN_TIMEOUT                               = 1002031,

            /** 登陆房间鉴权失败 */
            ZEGO_ERROR_CODE_ROOM_ERROR_AUTHENTICATION_FAILED                       = 1002033,

            /** 登陆房间的用户数超过该房间配置的最大用户数量限制 */
            ZEGO_ERROR_CODE_ROOM_ERROR_EXCEED_MAXIMUM_MEMBER                       = 1002034,

            /** 用户被踢出房间，可能是相同用户ID在其他设备登陆 */
            ZEGO_ERROR_CODE_ROOM_KICKED_OUT                                        = 1002050,

            /** 网络原因导致房间连接临时中断，正在重试 */
            ZEGO_ERROR_CODE_ROOM_CONNECT_TEMPORARY_BROKEN                          = 1002051,

            /** 房间连接断开，可能是由于网络原因导致 */
            ZEGO_ERROR_CODE_ROOM_DISCONNECT                                        = 1002052,

            /** 重试登录房间超过最大的重试时间 */
            ZEGO_ERROR_CODE_ROOM_RETRY_TIMEOUT                                     = 1002053,

            /** 业务后台发出了踢出房间信令，请查看被踢原因 */
            ZEGO_ERROR_CODE_ROOM_MANUAL_KICKED_OUT                                 = 1002055,

            /** 系统内部异常导致房间登陆失败 */
            ZEGO_ERROR_CODE_ROOM_INNER_ERROR                                       = 1002099,

            /** 推流失败，有可能是推流没有数据 */
            ZEGO_ERROR_CODE_PUBLISHER_PUBLISH_STREAM_FAILED                        = 1003001,

            /** 码率设置有误，请检查单位(kbps)是否有误 */
            ZEGO_ERROR_CODE_PUBLISHER_BITRATE_INVALID                              = 1003002,

            /** 推流流控参数设置有误 */
            ZEGO_ERROR_CODE_PUBLISHER_TRAFFIC_MODE_INVALID                         = 1003005,

            /** 推流临时中断，正在重试 */
            ZEGO_ERROR_CODE_PUBLISHER_ERROR_NETWORK_INTERRUPT                      = 1003020,

            /** 推流失败，该用户已在推流 */
            ZEGO_ERROR_CODE_PUBLISHER_ERROR_ALREADY_DO_PUBLISH                     = 1003023,

            /** 推流失败，该流被后台系统配置为禁止推送 */
            ZEGO_ERROR_CODE_PUBLISHER_ERROR_SERVER_FORBID                          = 1003025,

            /** 推流失败，房间内已有相同的流 */
            ZEGO_ERROR_CODE_PUBLISHER_ERROR_REPETITIVE_PUBLISH_STREAM              = 1003028,

            /** 更新转推CDN状态失败，请检测URL是否合法 */
            ZEGO_ERROR_CODE_PUBLISHER_UPDATE_CDN_TARGET_ERROR                      = 1003040,

            /** 发送SEI失败，data为空 */
            ZEGO_ERROR_CODE_PUBLISHER_SEI_DATA_NULL                                = 1003043,

            /** 发送SEI失败，data超长，最大支持4096个字符 */
            ZEGO_ERROR_CODE_PUBLISHER_SEI_DATA_TOO_LONG                            = 1003044,

            /** 流附加信息为空 */
            ZEGO_ERROR_CODE_PUBLISHER_EXTRA_INFO_NULL                              = 1003050,

            /** 流附加信息超长，最大为1024个字符 */
            ZEGO_ERROR_CODE_PUBLISHER_EXTRA_INFO_TOO_LONG                          = 1003051,

            /** 更新流附加信息失败，请检查网络连接 */
            ZEGO_ERROR_CODE_PUBLISHER_UPDATE_EXTRA_INFO_FAILED                     = 1003053,

            /** 推流水印路径为空 */
            ZEGO_ERROR_CODE_PUBLISHER_WATERMARK_URL_NULL                           = 1003055,

            /** 推流水印路径超长，最大为1024个字符 */
            ZEGO_ERROR_CODE_PUBLISHER_WATERMARK_URL_TOO_LONG                       = 1003056,

            /** 推流水印格式错误，仅支持 jpg 和 png 格式 */
            ZEGO_ERROR_CODE_PUBLISHER_WATERMARK_URL_INVALID                        = 1003057,

            /** 推流水印布局有误，布局区域不能超出编码分辨率大小 */
            ZEGO_ERROR_CODE_PUBLISHER_WATERMARK_LAYOUT_INVALID                     = 1003058,

            /** 系统内部异常导致推流失败 */
            ZEGO_ERROR_CODE_PUBLISHER_INNER_ERROR                                  = 1003099,

            /** 推流失败，有可能是拉流没有数据 */
            ZEGO_ERROR_CODE_PLAYER_PLAY_STREAM_FAILED                              = 1004001,

            /** 拉流超过最大的数量，目前支持最多12路拉流（如有需要可联系技术支持扩容） */
            ZEGO_ERROR_CODE_PLAYER_COUNT_EXCEED                                    = 1004010,

            /** 拉流临时中断，正在重试 */
            ZEGO_ERROR_CODE_PLAYER_ERROR_NETWORK_INTERRUPT                         = 1004020,

            /** 系统内部异常导致拉流失败 */
            ZEGO_ERROR_CODE_PLAYER_INNER_ERROR                                     = 1004099,

            /** 无混流服务，请联系技术支持开启 */
            ZEGO_ERROR_CODE_MIXER_NO_SERVICES                                      = 1005000,

            /** 混流任务ID为空 */
            ZEGO_ERROR_CODE_MIXER_TASK_ID_NULL                                     = 1005001,

            /** 混流任务ID超长，最大为256个字符 */
            ZEGO_ERROR_CODE_MIXER_TASK_ID_TOO_LONG                                 = 1005002,

            /** 混流任务ID中含有非法字符 */
            ZEGO_ERROR_CODE_MIXER_TASK_ID_INVALID_CHARACTER                        = 1005003,

            /** 混流任务配置未指定输出 */
            ZEGO_ERROR_CODE_MIXER_NO_OUTPUT_TARGET                                 = 1005005,

            /** 混流输出目标有误，请检查是否target为streamID时，传入了非法字符 */
            ZEGO_ERROR_CODE_MIXER_OUTPUT_TARGET_INVALID                            = 1005006,

            /** 启动混流任务请求失败，可能是由于网络原因 */
            ZEGO_ERROR_CODE_MIXER_START_REQUEST_ERROR                              = 1005010,

            /** 停止混流任务请求失败，可能是由于网络原因 */
            ZEGO_ERROR_CODE_MIXER_STOP_REQUEST_ERROR                               = 1005011,

            /** 该混流任务必须由该任务的启动用户执行停止操作 */
            ZEGO_ERROR_CODE_MIXER_NOT_OWNER_STOP_MIXER                             = 1005012,

            /** 混流任务启动过于频繁 */
            ZEGO_ERROR_CODE_MIXER_START_QPS_OVERLOAD                               = 1005015,

            /** 混流任务停止过于频繁 */
            ZEGO_ERROR_CODE_MIXER_STOP_QPS_OVERLOAD                                = 1005016,

            /** 混流任务输入流列表为空 */
            ZEGO_ERROR_CODE_MIXER_INPUT_LIST_INVALID                               = 1005020,

            /** 混流任务输出列表为空 */
            ZEGO_ERROR_CODE_MIXER_OUTPUT_LIST_INVALID                              = 1005021,

            /** 混流任务视频配置无效 */
            ZEGO_ERROR_CODE_MIXER_VIDEO_CONFIG_INVALID                             = 1005023,

            /** 混流任务音频配置无效，请检查是否使用了不支持的编解码 */
            ZEGO_ERROR_CODE_MIXER_AUDIO_CONFIG_INVALID                             = 1005024,

            /** 超过最大的输入流数量，最大支持9个输入流 */
            ZEGO_ERROR_CODE_MIXER_EXCEED_MAX_INPUT_COUNT                           = 1005025,

            /** 输入流不存在 */
            ZEGO_ERROR_CODE_MIXER_INPUT_STREAM_NOT_EXISTS                          = 1005026,

            /** 混流输入参数错误，可能是输入流的布局超过画布范围 */
            ZEGO_ERROR_CODE_MIXER_INPUT_PARAMETERS_ERROR                           = 1005027,

            /** 超过最大的输出流数量，最大支持3个输出流 */
            ZEGO_ERROR_CODE_MIXER_EXCEED_MAX_OUTPUT_COUNT                          = 1005030,

            /** 混流认证失败 */
            ZEGO_ERROR_CODE_MIXER_AUTHENTICATION_FAILED                            = 1005050,

            /** 输入水印为空 */
            ZEGO_ERROR_CODE_MIXER_WATERMARK_NULL                                   = 1005061,

            /** 输入水印参数错误，可能是布局超过画布范围 */
            ZEGO_ERROR_CODE_MIXER_WATERMARK_PARAMETERS_ERROR                       = 1005062,

            /** 输入水印URL非法，必须以preset-id://开头且需要是.jpg或.png结尾 */
            ZEGO_ERROR_CODE_MIXER_WATERMARK_URL_INVALID                            = 1005063,

            /** 输入背景图URL非法，必须以preset-id://开头且需要是.jpg或.png结尾 */
            ZEGO_ERROR_CODE_MIXER_BACKGROUND_IMAGE_URL_INVALID                     = 1005067,

            /** 未找到自动混流服务器，请联系技术支持配置 */
            ZEGO_ERROR_CODE_MIXER_AUTO_MIX_STREAM_SERVER_NOT_FOUND                 = 1005070,

            /** 混流内部错误 */
            ZEGO_ERROR_CODE_MIXER_INNER_ERROR                                      = 1005099,

            /** 设备一般错误 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_GENERIC                              = 1006001,

            /** 设备ID不存在 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_INVALID_ID                           = 1006002,

            /** 没有设备权限，请检查摄像头或麦克风的权限 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_NO_AUTHORIZATION                     = 1006003,

            /** 视频设备采集帧率为0 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_ZERO_FPS                             = 1006004,

            /** 设备被占用 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_IN_USE_BY_OTHER                      = 1006005,

            /** 设备被拔出 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_UNPLUGGED                            = 1006006,

            /** 设备需要重启 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_TYPE_REBOOT_REQUIRED                      = 1006007,

            /** 设备媒体丢失 */
            ZEGO_ERROR_CODE_DEVICE_ERROR_MEDIA_SERVICES_LOST                       = 1006008,

            /** 释放释放设备列表时，列表为空 */
            ZEGO_ERROR_CODE_DEVICE_FREE_DEVICE_LIST_NULL                           = 1006020,

            /** 设备内部错误 */
            ZEGO_ERROR_CODE_DEVICE_INNER_ERROR                                     = 1006099,

            /** 美颜参数非法，请检查传入的参数 */
            ZEGO_ERROR_CODE_PREPROCESS_BEAUTIFY_OPTION_INVALID                     = 1007005,

            /** 没有创建该媒体播放器实例 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_NO_INSTANCE                               = 1008001,

            /** 媒体播放器播放失败，未加载资源文件 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_NO_FILE_PATH                              = 1008003,

            /** 媒体播放器载入文件失败，不支持该格式文件 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_FILE_FORMAT_ERROR                         = 1008005,

            /** 媒体播放器载入文件失败，路径不存在 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_FILE_PATH_NOT_EXISTS                      = 1008006,

            /** 媒体播放器载入文件失败，解码错误 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_FILE_DECODE_ERROR                         = 1008007,

            /** 媒体播放器载入文件失败，无可播放的音视频流 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_FILE_NO_SUPPORTED_STREAM                  = 1008008,

            /** 媒体播放器播放失败，文件解析过程中出现错误 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_DEMUX_ERROR                               = 1008010,

            /** 媒体播放器Seek失败，可能是还没加载文件 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_SEEK_ERROR                                = 1008016,

            /** 媒体播放器设置了该平台不支持的视频数据格式（如iOS平台下 CVPixelBuffer 不支持NV21） */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_PLATFORM_FORMAT_NOT_SUPPORTED             = 1008020,

            /** 媒体播放器创建实例超过最大限制，最多可创建4个实例 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_EXCEED_MAX_COUNT                          = 1008030,

            /** 媒体播放器内部错误 */
            ZEGO_ERROR_CODE_MEDIA_PLAYER_INNER_ERROR                               = 1008099,

            /** 输入消息内容为空 */
            ZEGO_ERROR_CODE_IM_CONTENT_NULL                                        = 1009001,

            /** 输入消息内容超长，最长为1024字节 */
            ZEGO_ERROR_CODE_IM_CONTENT_TOO_LONG                                    = 1009002,

            /** 发送消息失败，可能是由于网络原因 */
            ZEGO_ERROR_CODE_IM_SEND_FAILED                                         = 1009010,

            /** 自定义视频采集器未创建，请确保在收到OnStart回调后使用 */
            ZEGO_ERROR_CODE_CUSTOM_VIDEO_IO_CAPTURER_NOT_CREATED                   = 1011001,

            /** 自定义视频采集模块未开启，请确保在初始化配置中已开启自定义视频采集 */
            ZEGO_ERROR_CODE_CUSTOM_VIDEO_IO_NO_CUSTOM_VIDEO_CAPTURE                = 1011002,

            /** 当前设置的自定义视频采集格式不支持该API */
            ZEGO_ERROR_CODE_CUSTOM_VIDEO_IO_NOT_SUPPORTED_FORMAT                   = 1011010,


        };

    } //namespace EXPRESS
} //namespace ZEGO

#endif // __ZEGOEXPRESSERRORCODE_H__
